#include <MP1241.h>
#include <MP1090.h>
#include <Wire.h>
#include <PinChangeInt.h>

const int Radio_PWR_Pin = 13;

const int MP1093_VOL_A = 14;
const int MP1093_VOL_B = 15;
const int MP1093_BAND_1_A = 16;
const int MP1093_BAND_1_B = 17;
const int MP1093_BAND_2_A = 2;
const int MP1093_BAND_2_B = 3;
const int MP1093_BAND_3_A = 4;
const int MP1093_BAND_3_B = 5;
const int MP1093_BAND_4_A = 6;
const int MP1093_BAND_4_B = 7;
const int MP1093_BASS_A = 8;
const int MP1093_BASS_B = 9;
const int MP1093_TREBLE_A = 10;
const int MP1093_TREBLE_B = 11;
const int MP1093_BUTTON = 12;

const int MP1093_CounterClockWiseRot = 0;
const int MP1093_ClockWiseRot = 1;
const int MP1093_Unchanged = 2;

volatile byte VOLUME_FLAG = MP1093_Unchanged;
volatile byte BAND_1_FLAG = MP1093_Unchanged;
volatile byte BAND_2_FLAG = MP1093_Unchanged;
volatile byte BAND_3_FLAG = MP1093_Unchanged;
volatile byte BAND_4_FLAG = MP1093_Unchanged;
volatile byte BASS_FLAG = MP1093_Unchanged;
volatile byte TREBLE_FLAG = MP1093_Unchanged;
volatile byte MP1093_BUTTON_FLAG = MP1093_Unchanged;

void setup()
{
  Serial.begin (9600);
  Wire.begin();
  
  PCintPort::attachInterrupt(MP1093_VOL_A, &MP1093_VOL_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_VOL_B, &MP1093_VOL_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_1_A, &MP1093_BAND_1_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_1_B, &MP1093_BAND_1_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_2_A, &MP1093_BAND_2_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_2_B, &MP1093_BAND_2_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_3_A, &MP1093_BAND_3_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_3_B, &MP1093_BAND_3_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_4_A, &MP1093_BAND_4_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BAND_4_B, &MP1093_BAND_4_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BASS_A, &MP1093_BASS_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BASS_B, &MP1093_BASS_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TREBLE_A, &MP1093_TREBLE_A_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_TREBLE_B, &MP1093_TREBLE_B_ISR, RISING);
  PCintPort::attachInterrupt(MP1093_BUTTON, &MP1093_BUTTON_ISR, RISING);
      
  MP1090::Init(Radio_PWR_Pin);
  MP1090::SetStation(95200);
  MP1090::SetVolume(4);
    
  MP1241::Init ();
}
//--------------------------------------------------------
void loop ()
{ 
  if ((VOLUME_FLAG == MP1093_ClockWiseRot) || (VOLUME_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeVolume ();
    Serial.print ("Volume   ");
    Serial.print (MP1241::GetParameter (MP1241_VOLUME), HEX);
    Serial.print ("\n");
    VOLUME_FLAG = MP1093_Unchanged;
  }
   
  if ((BAND_1_FLAG == MP1093_ClockWiseRot) || (BAND_1_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeFilter (MP1241_FR, BAND_1_FLAG);
    Serial.print ("BAND_1   ");
    Serial.print (MP1241::GetParameter (MP1241_FR), HEX);
    Serial.print ("\n");
    BAND_1_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_2_FLAG == MP1093_ClockWiseRot) || (BAND_2_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeFilter (MP1241_FL, BAND_2_FLAG);
    Serial.print ("BAND_2   ");
    Serial.print (MP1241::GetParameter (MP1241_FL), HEX);
    Serial.print ("\n");
    BAND_2_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_3_FLAG == MP1093_ClockWiseRot) || (BAND_3_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeFilter (MP1241_RR, BAND_3_FLAG);
    Serial.print ("BAND_3   ");
    Serial.print (MP1241::GetParameter (MP1241_RR), HEX);
    Serial.print ("\n");
    BAND_3_FLAG = MP1093_Unchanged;
  }
  
  if ((BAND_4_FLAG == MP1093_ClockWiseRot) || (BAND_4_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeFilter (MP1241_RL, BAND_4_FLAG);
    Serial.print ("BAND_4   ");
    Serial.print (MP1241::GetParameter (MP1241_RL), HEX);
    Serial.print ("\n");
    BAND_4_FLAG = MP1093_Unchanged;
  }
  
  if ((BASS_FLAG == MP1093_ClockWiseRot) || (BASS_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBassTreble (MP1241_BASS, BASS_FLAG);
    Serial.print ("BASS   ");
    Serial.print (MP1241::GetParameter (MP1241_BASS), HEX);
    Serial.print ("\n");
    BASS_FLAG = MP1093_Unchanged;
  }
  
  if ((TREBLE_FLAG == MP1093_ClockWiseRot) || (TREBLE_FLAG == MP1093_CounterClockWiseRot))
  {
    ChangeBassTreble (MP1241_TREBLE, TREBLE_FLAG);
    Serial.print ("TREBLE   ");
    Serial.print (MP1241::GetParameter (MP1241_TREBLE), HEX);
    Serial.print ("\n");
    TREBLE_FLAG = MP1093_Unchanged;
  }
  
  if (MP1093_BUTTON_FLAG == MP1093_ClockWiseRot)
  {
    ChangeInput ();
    Serial.print ("Input   ");
    Serial.print ((0x8 - MP1241::GetParameter (MP1241_SWITCH)), HEX);
    Serial.print ("\n");
    MP1093_BUTTON_FLAG = MP1093_Unchanged;
  }
}
//--------------------------------------------------------
void MP1093_VOL_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_A)
  {
   delay (100);
   
   if ((digitalRead (MP1093_VOL_A)) && (!digitalRead (MP1093_VOL_B)))
   {
     VOLUME_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_VOL_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_VOL_B)
  {
    delay (100);
    if ((digitalRead (MP1093_VOL_B)) && (!digitalRead (MP1093_VOL_A)))
    {
      VOLUME_FLAG = MP1093_CounterClockWiseRot;
    }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_1_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_1_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_1_A)) && (!digitalRead (MP1093_BAND_1_B)))
     {
        BAND_1_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_1_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_1_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_1_B)) && (!digitalRead (MP1093_BAND_1_A)))
   {
     BAND_1_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_2_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_2_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_2_A)) && (!digitalRead (MP1093_BAND_2_B)))
     {
        BAND_2_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_2_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_2_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_2_B)) && (!digitalRead (MP1093_BAND_2_A)))
   {
     BAND_2_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_3_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_3_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_3_A)) && (!digitalRead (MP1093_BAND_3_B)))
     {
        BAND_3_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_3_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_3_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_3_B)) && (!digitalRead (MP1093_BAND_3_A)))
   {
     BAND_3_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_4_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_4_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BAND_4_A)) && (!digitalRead (MP1093_BAND_4_B)))
     {
        BAND_4_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BAND_4_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BAND_4_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BAND_4_B)) && (!digitalRead (MP1093_BAND_4_A)))
   {
     BAND_4_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BASS_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BASS_A)
  {
     delay (100);
     if ((digitalRead (MP1093_BASS_A)) && (!digitalRead (MP1093_BASS_B)))
     {
        BASS_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_BASS_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BASS_B)
  {
   delay (100);
   if ((digitalRead (MP1093_BASS_B)) && (!digitalRead (MP1093_BASS_A)))
   {
     BASS_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_TREBLE_A_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TREBLE_A)
  {
     delay (100);
     if ((digitalRead (MP1093_TREBLE_A)) && (!digitalRead (MP1093_TREBLE_B)))
     {
        TREBLE_FLAG = MP1093_ClockWiseRot;
     }
  }
}
//-----------------------------------------------------------------
void MP1093_TREBLE_B_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_TREBLE_B)
  {
   delay (100);
   if ((digitalRead (MP1093_TREBLE_B)) && (!digitalRead (MP1093_TREBLE_A)))
   {
     TREBLE_FLAG = MP1093_CounterClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void MP1093_BUTTON_ISR ()
{
  if (PCintPort::arduinoPin == MP1093_BUTTON)
  {
   delay (100);
   if ((digitalRead (MP1093_BUTTON)))
   {
     MP1093_BUTTON_FLAG = MP1093_ClockWiseRot;
   }
  }
}
//-----------------------------------------------------------------
void ChangeVolume ()
{ 
  byte CurValue = MP1241::GetParameter (MP1241_VOLUME);
  byte NewValue = CurValue;
  
  if (VOLUME_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue < 0x3F)
      NewValue++;
  }
  
  else if (VOLUME_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue > 0x0C)
     NewValue--;
  }
  
  if (NewValue != CurValue)
    MP1241::SetParameter (MP1241_VOLUME, NewValue);
}
//-----------------------------------------------------------------
void ChangeFilter (byte Filter, byte FILTER_FLAG)
{
  byte CurValue = MP1241::GetParameter (Filter);
  byte NewValue = CurValue;

  if (FILTER_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue < 0x3F)
      NewValue++;
  }
  
  else if (FILTER_FLAG == MP1093_CounterClockWiseRot)
  {
    if (NewValue > 0x07)
      NewValue--;
  }
  
  if (NewValue != CurValue)
    MP1241::SetParameter (Filter, NewValue);
}
//-----------------------------------------------------------------
void ChangeBassTreble (byte Filter, byte FILTER_FLAG)
{
  byte CurValue = MP1241::GetParameter (Filter);
  byte NewValue = CurValue;

  if (FILTER_FLAG == MP1093_ClockWiseRot)
  {
    if (NewValue < 0x1F)
      NewValue++;
  }
  
  else if (FILTER_FLAG == MP1093_CounterClockWiseRot)
  {
    if (((Filter == MP1241_BASS) && (NewValue > 0x04)) || ((Filter == MP1241_TREBLE) && (NewValue > 0x07)))
      NewValue--;
  }
  
  if (NewValue != CurValue)
    MP1241::SetParameter (Filter, NewValue);
}
//-----------------------------------------------------------------
void ChangeInput ()
{
  byte CurValue = MP1241::GetParameter (MP1241_SWITCH);
  byte NewValue = CurValue;

  if (NewValue > 0x3)
    NewValue--;
  
  else if (NewValue == 0x3)
    NewValue = 0x7;
  
  if (NewValue != CurValue)
    MP1241::SelectInput (NewValue);
}
